/******************************************************************************/
// Bullfrog Engine Emulation Library - for use to remake classic games like
// Syndicate Wars, Magic Carpet or Dungeon Keeper.
/******************************************************************************/
// Author:  Tomasz Lis
// Created: 10 Feb 2008

// Purpose:
//    Keyboard related routines - reading keyboard.

// Comment:
//   None yet.

//Copying and copyrights:
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
/******************************************************************************/
#include "bflib_keybrd.h"

#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <SDL/SDL.h>
#include "bflib_mouse.h"

#ifdef __cplusplus
extern "C" {
#endif
/******************************************************************************/
unsigned char lbKeyOn[256];
unsigned char lbShift;
unsigned char lbIInkeyFlags;
unsigned char lbInkey;
unsigned char lbIInkey;
unsigned char lbInkeyFlags;
unsigned short flow_control_flags;
unsigned long text_buf_pos;
bool lbExtendedKeyPress=false;

const char AsciiToInkey[] = {
   0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
   0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
   0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
   0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
  0x39, 0x2, 0x3,0x2B, 0x5, 0x6, 0x8,0x28,
  0x0A,0x0B, 0x9,0x4E,0x33,0x0C,0x34,0x35,
  0x0B, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8,
   0x9,0x0A,0x27,0x27,0x33,0x0D,0x34,0x35,
  0x28,0x1E,0x30,0x2E,0x20,0x12,0x21,0x22,
  0x23,0x17,0x24,0x25,0x26,0x32,0x31,0x18,
  0x19,0x10,0x13,0x1F,0x14,0x16,0x2F,0x11,
  0x2D,0x15,0x2C,0x1A,0x56,0x1B, 0x7,0x0C,
  0x29,0x1E,0x30,0x2E,0x20,0x12,0x21,0x22,
  0x23,0x17,0x24,0x25,0x26,0x32,0x31,0x18,
  0x19,0x10,0x13,0x1F,0x14,0x16,0x2F,0x11,
  0x2D,0x15,0x2C,0x1A,0x56,0x1B,0x2B, 0x0,
};

char lbInkeyToAscii[] = {
  0x0, 0x0,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x30,0x2D,
 0x3D, 0x8, 0x9,0x71,0x77,0x65,0x72,0x74,0x79,0x75,0x69,0x6F,0x70,
 0x5B,0x5D, 0x0, 0x0,0x61,0x73,0x64,0x66,0x67,0x68,0x6A,0x6B,0x6C,
 0x3B,0x27,0x60, 0x0,0x23,0x7A,0x78,0x63,0x76,0x62,0x6E,0x6D,0x2C,
 0x2E,0x2F, 0x0,0x2A, 0x0,0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,0x2D, 0x0, 0x0, 0x0,
 0x2B, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,0x5C, 0x0, 0x0, 0x0, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0,0x2F, 0x0, 0x0,0x28,0x29,0x2F,0x2A, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,0x2E, 0x0, 0x0, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
};

char lbInkeyToAsciiShift[] = {
  0x0, 0x0,0x21,0x22,0x9C,0x24,0x25,0x5E,0x26,0x2A,0x28,0x29,0x5F,
 0x2B, 0x8, 0x9,0x51,0x57,0x45,0x52,0x54,0x59,0x55,0x49,0x4F,0x50,
 0x7B,0x7D, 0x0, 0x0,0x41,0x53,0x44,0x46,0x47,0x48,0x4A,0x4B,0x4C,
 0x3A,0x40,0x7E, 0x0,0x7E,0x5A,0x58,0x43,0x56,0x42,0x4E,0x4D,0x3C,
 0x3E,0x3F, 0x0,0x2A, 0x0,0x20, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,0x2D, 0x0, 0x0, 0x0,
 0x2B, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,0x7C, 0x0, 0x0, 0x0, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0,0x2F, 0x0, 0x0,0x28,0x29,0x2F,0x2A, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,0x2E, 0x0, 0x0, 0x0,
  0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
};

/******************************************************************************/
void KInt(void)
{
  static unsigned char oinkey=0;
  lbInkey=0;//_GETDS();
  unsigned int v1;
  if (oinkey==0xE0)
  {
    lbExtendedKeyPress=false;
    v1 = lbInkey&0x7F;
    lbKeyOn[v1] = (lbInkey<=0x7F); //lbInkey<=0x7F means the key was pressed, not released
  } else
  {
    lbExtendedKeyPress=true;
    if ((lbInkey!=0x2a)&&(lbInkey!=0xaa))
    {
      v1 = lbInkey|0x80;
      lbKeyOn[v1] = (lbInkey<=0x7F);
    } else
    {
      lbInkey = 0x80;
      v1 = 0x80;
    }
  }
  oinkey=lbInkey;
  lbInkeyFlags=0;
  if ((lbKeyOn[0x2a]!=0)||(lbKeyOn[0x36]!=0))
    lbInkeyFlags|=0x10;
  if ((lbKeyOn[0x1d]!=0)||(lbKeyOn[0x9d]!=0))
    lbInkeyFlags|=0x20;
  if ((lbKeyOn[0x38]!=0)||(lbKeyOn[0xb8]!=0))
    lbInkeyFlags|=0x40;
  if (lbKeyOn[v1]!=0)
    lbKeyOn[v1]|=lbInkeyFlags;
/*
  ... some direct keyboard control code here
  changes v1, bot without storing in global variables.
*/
}

void __fastcall LbStoreKeyStoke(SDL_KeyboardEvent *key)
{
  unsigned short key_idx=0;
  //Retrieving the keyboard Hardware keycode
  switch (key->keysym.sym)
  {
  case SDLK_ESCAPE:
       key_idx=KC_ESCAPE;
       break;
  case SDLK_1:
       key_idx=KC_1;
       break;
  case SDLK_2:
       key_idx=KC_2;
       break;
  case SDLK_3:
       key_idx=KC_3;
       break;
  case SDLK_4:
       key_idx=KC_4;
       break;
  case SDLK_5:
       key_idx=KC_5;
       break;
  case SDLK_6:
       key_idx=KC_6;
       break;
  case SDLK_7:
       key_idx=KC_7;
       break;
  case SDLK_8:
       key_idx=KC_8;
       break;
  case SDLK_9:
       key_idx=KC_9;
       break;
  case SDLK_0:
       key_idx=KC_0;
       break;
  case SDLK_MINUS:
       key_idx=KC_MINUS;    // - on main keyboard
       break;
  case SDLK_EQUALS:
       key_idx=KC_EQUALS;
       break;
  case SDLK_BACKSPACE:
       key_idx=KC_BACK;    // backspace
       break;
  case SDLK_TAB:
       key_idx=KC_TAB;
       break;
  case SDLK_q:
       key_idx=KC_Q;
       break;
  case SDLK_w:
       key_idx=KC_W;
       break;
  case SDLK_e:
       key_idx=KC_E;
       break;
  case SDLK_r:
       key_idx=KC_R;
       break;
  case SDLK_t:
       key_idx=KC_T;
       break;
  case SDLK_y:
       key_idx=KC_Y;
       break;
  case SDLK_u:
       key_idx=KC_U;
       break;
  case SDLK_i:
       key_idx=KC_I;
       break;
  case SDLK_o:
       key_idx=KC_O;
       break;
  case SDLK_p:
       key_idx=KC_P;
       break;
  case SDLK_LEFTBRACKET:
       key_idx=KC_LBRACKET;
       break;
  case SDLK_RIGHTBRACKET:
       key_idx=KC_RBRACKET;
       break;
  case SDLK_CARET:
       key_idx=KC_RETURN;    // Enter on main keyboard
       break;
  case SDLK_LCTRL:
       key_idx=KC_LCONTROL;
       break;
  case SDLK_a:
       key_idx=KC_A;
       break;
  case SDLK_s:
       key_idx=KC_S;
       break;
  case SDLK_d:
       key_idx=KC_D;
       break;
  case SDLK_f:
       key_idx=KC_F;
       break;
  case SDLK_g:
       key_idx=KC_G;
       break;
  case SDLK_h:
       key_idx=KC_H;
       break;
  case SDLK_j:
       key_idx=KC_J;
       break;
  case SDLK_k:
       key_idx=KC_K;
       break;
  case SDLK_l:
       key_idx=KC_L;
       break;
  case SDLK_SEMICOLON:
       key_idx=KC_SEMICOLON;
       break;
/*
  case SDLK_:
       key_idx=KC_APOSTROPHE;
       break;
  case SDLK_:
       key_idx=KC_GRAVE;    // accent
       break;
*/
  case SDLK_LSHIFT:
       key_idx=KC_LSHIFT;
       break;
  case SDLK_BACKSLASH:
       key_idx=KC_BACKSLASH;
       break;
  case SDLK_z:
       key_idx=KC_Z;
       break;
  case SDLK_x:
       key_idx=KC_X;
       break;
  case SDLK_c:
       key_idx=KC_C;
       break;
  case SDLK_v:
       key_idx=KC_V;
       break;
  case SDLK_b:
       key_idx=KC_B;
       break;
  case SDLK_n:
       key_idx=KC_N;
       break;
  case SDLK_m:
       key_idx=KC_M;
       break;
  case SDLK_COMMA:
       key_idx=KC_COMMA;
       break;
  case SDLK_PERIOD:
       key_idx=KC_PERIOD;    // . on main keyboard
       break;
  case SDLK_SLASH:
       key_idx=KC_SLASH;    // / on main keyboard
       break;
  case SDLK_RSHIFT:
       key_idx=KC_RSHIFT;
       break;
  case SDLK_KP_MULTIPLY:
       key_idx=KC_MULTIPLY;    // * on numeric keypad
       break;
  case SDLK_LALT:
       key_idx=KC_LALT;    // left Alt
       break;
  case SDLK_SPACE:
       key_idx=KC_SPACE;
       break;
  case SDLK_CAPSLOCK:
       key_idx=KC_CAPITAL;
       break;
  case SDLK_F1:
       key_idx=KC_F1;
       break;
  case SDLK_F2:
       key_idx=KC_F2;
       break;
  case SDLK_F3:
       key_idx=KC_F3;
       break;
  case SDLK_F4:
       key_idx=KC_F4;
       break;
  case SDLK_F5:
       key_idx=KC_F5;
       break;
  case SDLK_F6:
       key_idx=KC_F6;
       break;
  case SDLK_F7:
       key_idx=KC_F7;
       break;
  case SDLK_F8:
       key_idx=KC_F8;
       break;
  case SDLK_F9:
       key_idx=KC_F9;
       break;
  case SDLK_F10:
       key_idx=KC_F10;
       break;
  case SDLK_NUMLOCK:
       key_idx=KC_NUMLOCK;
       break;
  case SDLK_SCROLLOCK:
       key_idx=KC_SCROLL;    // Scroll Lock
       break;
  case SDLK_KP7:
       key_idx=KC_NUMPAD7;
       break;
  case SDLK_KP8:
       key_idx=KC_NUMPAD8;
       break;
  case SDLK_KP9:
       key_idx=KC_NUMPAD9;
       break;
  case SDLK_KP_MINUS:
       key_idx=KC_SUBTRACT;    // - on numeric keypad
       break;
  case SDLK_KP4:
       key_idx=KC_NUMPAD4;
       break;
  case SDLK_KP5:
       key_idx=KC_NUMPAD5;
       break;
  case SDLK_KP6:
       key_idx=KC_NUMPAD6;
       break;
  case SDLK_KP_PLUS:
       key_idx=KC_ADD;    // + on numeric keypad
       break;
  case SDLK_KP1:
       key_idx=KC_NUMPAD1;
       break;
  case SDLK_KP2:
       key_idx=KC_NUMPAD2;
       break;
  case SDLK_KP3:
       key_idx=KC_NUMPAD3;
       break;
  case SDLK_KP0:
       key_idx=KC_NUMPAD0;
       break;
  case SDLK_KP_PERIOD:
       key_idx=KC_DECIMAL;    // . on numeric keypad
       break;
/*
  case SDLK_:
       key_idx=KC_OEM_102;    // < > | on UK/Germany keyboards
       break;
*/
  case SDLK_F11:
       key_idx=KC_F11;
       break;
  case SDLK_F12:
       key_idx=KC_F12;
       break;
  case SDLK_F13:
       key_idx=KC_F13;
       break;
  case SDLK_F14:
       key_idx=KC_F14;
       break;
  case SDLK_F15:
       key_idx=KC_F15;
       break;
/*
  case SDLK_:
       key_idx=KC_KANA;
       break;
  case SDLK_:
       key_idx=KC_ABNT_C1;
       break;
  case SDLK_:
       key_idx=KC_CONVERT;
       break;
  case SDLK_:
       key_idx=KC_NOCONVERT;
       break;
  case SDLK_:
       key_idx=KC_YEN;
       break;
  case SDLK_:
       key_idx=KC_ABNT_C2;
       break;
*/
  case SDLK_KP_EQUALS:
       key_idx=KC_NUMPADEQUALS;
       break;
/*
  case SDLK_:
       key_idx=KC_PREVTRACK;
       break;
  case SDLK_:
       key_idx=KC_AT;
       break;
*/
  case SDLK_COLON:
       key_idx=KC_COLON;
       break;
/*
  case SDLK_:
       key_idx=KC_UNDERLINE;
       break;
  case SDLK_:
       key_idx=KC_KANJI;
       break;
  case SDLK_:
       key_idx=KC_STOP;
       break;
  case SDLK_:
       key_idx=KC_AX;
       break;
  case SDLK_:
       key_idx=KC_UNLABELED;
       break;
  case SDLK_:
       key_idx=KC_NEXTTRACK;
       break;
*/
  case SDLK_KP_ENTER:
       key_idx=KC_NUMPADENTER;
       break;
  case SDLK_RCTRL:
       key_idx=KC_RCONTROL;
       break;

/*TODO: add more keys here
*/
  default:
       key_idx=KC_UNASSIGNED;
       break;
  }
  lbKeyOn[key_idx]=(key->type==SDL_KEYDOWN);
  //Setting flags
  lbInkeyFlags=0;
  if ((lbKeyOn[KC_LSHIFT]!=0)||(lbKeyOn[KC_RSHIFT]!=0))
    lbInkeyFlags|=0x10;
  if ((lbKeyOn[KC_LCONTROL]!=0)||(lbKeyOn[KC_RCONTROL]!=0))
    lbInkeyFlags|=0x20;
  if ((lbKeyOn[KC_LALT]!=0)||(lbKeyOn[KC_RALT]!=0))
    lbInkeyFlags|=0x40;
  //Storing the keystroke
  if (lbKeyOn[key_idx])
    lbKeyOn[key_idx]|=lbInkeyFlags;
}
/*
inline void __fastcall LbProcessAllEvents()
{
  SDL_Event event;
  while(SDL_PollEvent(&event))
  {
    switch(event.type)
    {
    case SDL_KEYDOWN:
    case SDL_KEYUP:
      LbStoreKeyStoke(&event.key);
      break;
    case SDL_MOUSEMOTION:
      LbProcessMouseMove(&event.motion);
      break;
    case SDL_MOUSEBUTTONDOWN:
    case SDL_MOUSEBUTTONUP:
      LbProcessMouseClick(&event.button);
      break;
    case SDL_QUIT:
      flow_control_flags|=1;
      break;
    }
  }
}
*/

int __fastcall LbIKeyboardOpen(void)
{
  memset(lbKeyOn, 0, 256);
/* Setting interrupt should be replaced by SDL initation
  void *v1; // edx@1
  OldInt = dos_getvect(9);
  word_200510 = v1;
  dos_setvect(9,v1);
*/
  return 1;
}

int __fastcall LbIKeyboardClose(void)
{
/* Setting interrupt should be replaced by SDL initation
  void *edx0; // edx@0
  dos_setvect_(9, edx0);
*/
  return 1;
}

char __fastcall LbDosKeyboard()
{
  char result;
  result=0;
/* Calling interrupt should be replaced by SDL
  char v1; // [sp+1h] [bp-1Bh]@1
  char regs; // [sp+0h] [bp-1Ch]@1
  v1 = 18;
  result = int386_();
  lbShift = 0;
  if ( regs & 3 )
    lbShift = 1;
  if ( regs & 4 )
    lbShift = (unsigned __int8)lbShift | 2;
  if ( regs & 8 )
    lbShift = (unsigned __int8)lbShift | 4;
  lbInkey = 0;
  __asm { int     16h             ; KEYBOARD - CHECK ENHANCED KEYSTROKE (AT model 339,XT2,XT286,PS) }
*/
  return result;
}

char __fastcall LbKeyboard()
{
  char result;
  LbDosKeyboard();
  if ( ((unsigned char)lbInkey) >= 128 )
    result = 0;
  else
    result = lbInkeyToAscii[lbInkey];
  return result;
}
/*
int __fastcall control_process(void)
{
  LbProcessAllEvents();
  if ( lbKeyOn[0x2a] || lbKeyOn[0x36] )
    lbShift |= 1;
  else
    lbShift &= 0x0FEu;
  if ( lbKeyOn[0x1d] )
    lbShift |= 0x02;
  else
    lbShift &= 0x0FDu;
  if ( lbKeyOn[0x0b8] || lbKeyOn[0x38] )
  {
    lbShift |= 0x04;
    if ( lbInkey == 45 )
    {
      flow_control_flags |= 0x0001;
      lbInkey = 0;
    }
  }
  else
  {
    lbShift &= 0x0FBu;
  }
  return 0;
}
*/
/******************************************************************************/
#ifdef __cplusplus
}
#endif
